import 'dart:convert';
import 'package:loyalty_management_tool/bloc/common_widgets/loading_indicator.dart';
import 'package:loyalty_management_tool/bloc/common_widgets/paginator_list_view.dart';
import 'package:loyalty_management_tool/bloc/common_widgets/rewards_widget.dart';
import 'package:loyalty_management_tool/bloc/cubits/cubit_main.dart';
import 'package:loyalty_management_tool/bloc/states/main_state.dart';
import 'package:loyalty_management_tool/bloc/states/states.dart';
import 'package:loyalty_management_tool/models/create_post_payload.dart';
import 'package:loyalty_management_tool/models/rewards_list_response.dart';
import 'package:loyalty_management_tool/utils/appAvatar.dart';

import 'package:loyalty_management_tool/utils/basicInfo.dart';
import 'package:loyalty_management_tool/utils/colors.dart';
import 'package:loyalty_management_tool/utils/custom_app_bar.dart';
import 'package:loyalty_management_tool/utils/customcard.dart';
import 'package:loyalty_management_tool/utils/empty_widget.dart';
import 'package:loyalty_management_tool/utils/hexColors.dart';
import 'package:loyalty_management_tool/utils/localization.dart';
import 'package:loyalty_management_tool/utils/locator.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:loyalty_management_tool/utils/resolutionenums.dart';
import 'package:loyalty_management_tool/utils/serviceTypeEnums.dart';
import 'package:loyalty_management_tool/utils/text_styling.dart';

import 'package:shared_preferences/shared_preferences.dart';

import '../../../models/couponlisting.dart';
import 'loyalty_type_page_new.dart';

class LeadgerHistoryPage extends StatefulWidget {


  LeadgerHistoryPage();

  @override
  State<StatefulWidget> createState() => LeadgerHistoryPageState();
}

class LeadgerHistoryPageState extends State<LeadgerHistoryPage> {
  final scrollController = ScrollController();
  late TextStyleElements styleElements;
  List<dynamic> list = [];
  bool isLastPage = false;
  bool isActive = true;
  String? searchValue;
  BasicInfo? basicInfo = BasicInfo();
  List<PopupMenuItem> menuList = [];
  SharedPreferences? prefs = locatorRewards<SharedPreferences>();
  late BuildContext ctx;

  @override
  void initState() {
    setupScrollController();
    WidgetsBinding.instance!.addPostFrameCallback((_) {
      BlocProvider.of<CubitMain>(context)
          .loadPage(getBody(), context, basicInfo!.couponLeaderList, getData);
    });

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    this.ctx = context;
    styleElements = TextStyleElements(context);

    return Scaffold(
      body: _getBody(),
    );
  }

  Widget _getBody() {
    return BlocBuilder<CubitMain, MainState>(builder: (context, state) {
      styleElements = TextStyleElements(context);

      if (state is LoadingStatePagination && state.isFirstFetch) {
        return LoadingIndicator();
      }
      bool isLoading = false;
      if (state is LoadingStatePagination) {
        list = state.list;
        isLoading = true;
      } else if (state is Loaded) {
        list = state.list;
        isLastPage = state.isLastPage;
      } else if (state is Empty) {
        return Center(
          child: EmptyWidget(
              message:
                  AppLocalizationsRewards.of(context)!.translate("no_data")),
        );
      }
      return Padding(
        padding: const EdgeInsets.only(top: 8.0),
        child: PaginatorListView(
          scrollController: scrollController,
          list: list,
          isLoading: isLoading,
          itemBuilder: getPostWidget,
        ),
      );
    });
  }

  Widget getPostWidget(value, index) {
    CouponHistoryItem item = value;

    return ListTile(
      leading: appAvatar(imageUrl: '',),
      title: Text(item.transactionType ?? "-"),
      subtitle: Text(item.transactionDatetime ?? "-"),
      trailing: Text(item.lineBalance ?? "-"),
    );
  }

  dynamic getData(dynamic data) {
    return CouponHistoryResponse.fromJson(data);
  }

  String getBody() {
    CouponListRequest request = CouponListRequest();
    request.businessId = basicInfo!.businessId.toString();
    request.objectType = "business";
    request.personId = basicInfo!.userId.toString();
    request.pageNumber = BlocProvider.of<CubitMain>(ctx).page;
    request.pageSize = 10;
    return jsonEncode(request);
  }

  void setupScrollController() {
    scrollController.addListener(() {
      if (scrollController.position.atEdge) {
        if (scrollController.position.pixels != 0) {
          if (!isLastPage) {
            BlocProvider.of<CubitMain>(ctx).loadPage(
                getBody(), ctx, basicInfo!.loyaltyProgramList, getData);
          }
        }
      }
    });
  }
}
